import hypermedia.video.*;
import java.awt.*;


import oscP5.*;
import netP5.*;

OscP5 oscP5;
NetAddress myRemoteLocation;


OpenCV opencv;

int w = 320;
int h = 240;
int threshold = 80;
float k=0;
final int COLOR_SPACE = OpenCV.RGB;

boolean find=true;

PFont font;

void setup() {

  size( w*2+30, h*2+30 );

  oscP5 = new OscP5(this,12000);

  opencv = new OpenCV( this );
  opencv.capture(w,h);

  font = loadFont( "AndaleMono.vlw" );
  textFont( font );

  println( "Drag mouse inside sketch window to change threshold" );
  println( "Press space bar to record background image" );

  myRemoteLocation = new NetAddress("192.168.2.10",12000);
}



void draw() {

  background(0);
  opencv.read();
  //opencv.flip( OpenCV.FLIP_HORIZONTAL );
  OscMessage myMessage = new OscMessage("/blob/data/Rect");
  OscMessage myMessagePointsX = new OscMessage("/blob/data/PointsX");
  OscMessage myMessagePointsY = new OscMessage("/blob/data/PointsY");
  if ( COLOR_SPACE!=OpenCV.RGB ) opencv.convert( COLOR_SPACE );
  opencv.restore( COLOR_SPACE );
  opencv.blur( OpenCV.GAUSSIAN, 30 );
  image( opencv.image(), 10, 10 );
  image( opencv.image(), 10, 10 );	            // RGB image
  image( opencv.image(OpenCV.GRAY), 20+w, 10 );   // GRAY image
  image( opencv.image(OpenCV.MEMORY), 10, 20+h ); // image in memory

  opencv.absDiff();
  opencv.threshold(threshold);
  image( opencv.image(OpenCV.GRAY), 20+w, 20+h ); // absolute difference image


  // working with blobs
  Blob[] blobs = opencv.blobs( 100, w*h/3, 20, true );

  noFill();

  pushMatrix();
  translate(20+w,20+h);
  k=0;
  for( int i=0; i<blobs.length; i++ ) {

    Rectangle bounding_rect	= blobs[i].rectangle;
    float area = blobs[i].area;
    float circumference = blobs[i].length;
    Point centroid = blobs[i].centroid;
    Point[] points = blobs[i].points;

    if (blobs[i].isHole==false)
    {
      myMessage.add(new float[] {
        blobs[i].rectangle.x,blobs[i].rectangle.y,blobs[i].rectangle.width,blobs[i].rectangle.height
      }
      );
    }
    // rectangle
    noFill();
    stroke( blobs[i].isHole ? 128 : 64 );
    rect( bounding_rect.x, bounding_rect.y, bounding_rect.width, bounding_rect.height );


    // centroid
    stroke(0,0,255);
    line( centroid.x-5, centroid.y, centroid.x+5, centroid.y );
    line( centroid.x, centroid.y-5, centroid.x, centroid.y+5 );
    noStroke();
    fill(0,0,255);
    text(bounding_rect.x,centroid.x+5, centroid.y+5 );


    fill(255,0,255,64);
    stroke(255,0,255);
    if ( points.length>0 ) {
      beginShape();
      for( int j=0; j<points.length; j++ ) {
        vertex( points[j].x, points[j].y );
        if (blobs[i].isHole==false && k<blobs[i].area)
        {
          if (j==0)
          {
            myMessagePointsX=new OscMessage("/blob/data/PointsX");
            ;
            myMessagePointsY=new OscMessage("/blob/data/PointsY");
            ;
          }
          myMessagePointsX.add(points[j].x );
          myMessagePointsY.add(points[j].y );
        }
      }
      endShape(CLOSE);
    }
    if (k<blobs[i].area) {
      k=blobs[i].area;
    }
    noStroke();
    fill(255,0,255);
    text( circumference, centroid.x+5, centroid.y+15 );
  }
  popMatrix();
  oscP5.send(myMessage, myRemoteLocation);
  oscP5.send(myMessagePointsX, myRemoteLocation);
  oscP5.send(myMessagePointsY, myRemoteLocation);
}

void keyPressed() {
  if ( key==' ' ) opencv.remember();
}

void mouseDragged() {
  threshold = int( map(mouseX,0,width,0,255) );
}

public void stop() {
  opencv.stop();
  super.stop();
}

